# import the HyPhy library
# and standard OS utilities

import os, HyPhy

# first, create a HyPhy interface instance (class _THyPhy)
# the first argument defines the root directory for HyPhy
# and the second - how many threads the computational core
# should spawn

hyphyInstance = HyPhy._THyPhy (os.getcwd(),2)

# the basic interface command is 'ExecuteBF' which
# executes HyPhy batch language commands in HyPhy
# and returns a string representation of the return value
# (if any) from HYPHY
# The returned object is of type _THyPhyString with 
# sData and sLength fields
# HyPhy will take care of disposing of the memory needed 
# to store the result

hyphyResult = hyphyInstance.ExecuteBF ("return 2+2;");
print ("Testing a trivial HyPhy command. 2+2 = ", hyphyResult.sData)

# an optional second argument to ExecuteBF
# can be used to "flush" the current state of the system

# this is the default option for the call of ExecuteBF
# passing the second argument of False or 0 will preserve
# the execution state 

print ("Consecutive command exection")
hyphyInstance.ExecuteBF ("z:=x+y;",False);
hyphyInstance.ExecuteBF ("x=3;",False);
hyphyInstance.ExecuteBF ("y=5;",False);
hyphyResult = hyphyInstance.ExecuteBF ("return z;",False);
print ("The value of z is ", hyphyResult.sData)

print ("Resetting the state of the execution erases the value of 'z'")
hyphyResult = hyphyInstance.ExecuteBF ("return z;");
print ("The value of z is ", hyphyResult.sData)

# the real utility of the interface is to be able 
# to execute prewritten analyses from HBL files

print ("Executing the example F81.bf file")
hyphyInstance.ExecuteBF ("ExecuteAFile(\"../HBL/F81.bf\")");

# retrive the standard output, error and runtime warnings

hyphyOut 		= hyphyInstance.GetStdout()
#errors will be empty UNLESS there was an exection error
hyphyErrors     = hyphyInstance.GetErrors()
hyphyWarnings	= hyphyInstance.GetWarnings()

print ("Standard out: \n", hyphyOut.sData)
print ("Errors: \n", hyphyErrors.sData)
print ("Warnings/Log messages: \n", hyphyWarnings.sData)

# these variables can be explicitly deleted when they are no longer needed
# python garbage collection should take care of disposing of disused variables

del hyphyOut
del hyphyErrors
del hyphyWarnings

# A tighter intergration can be achieved by defining a retrieval function 
# with the reserved name _THyPhyAskFor in the HBL file; it retrieves data
# by key and returns them in a internal format that can be converted 
# to one of the basic return types: number, string or matrix

def retrieveValueByKey (key, returnType,hyphyInstance):
	theResult = hyphyInstance.AskFor(key)
	# see if HyPhy can retrieve a value with the requested key
	if theResult:
   		canICast = hyphyInstance.CanCast(theResult,returnType)
   		# see if HyPhy can cast the value to the requested type
   		if canICast:
   			# do the casting
   			theResult = hyphyInstance.CastResult(theResult,returnType)
   			# the last step is to convert from the basic return type
   			# to a derived class that we can use in python directly
   			if (returnType == HyPhy.THYPHY_TYPE_NUMBER):
   				return theResult.castToNumber()
   			if (returnType == HyPhy.THYPHY_TYPE_STRING):
   				return theResult.castToString()
   			if (returnType == HyPhy.THYPHY_TYPE_MATRIX):
   				return theResult.castToMatrix()
	return null   		
   		

hyphyResult = hyphyInstance.ExecuteBF ("ExecuteAFile(\"../HBL/HKY85.bf\")");


print ("Log-L = ", retrieveValueByKey ("LogL", HyPhy.THYPHY_TYPE_NUMBER, hyphyInstance).nValue)
print ("kappa = ", retrieveValueByKey ("kappa", HyPhy.THYPHY_TYPE_NUMBER, hyphyInstance).nValue)
print ("tree string = ", retrieveValueByKey ("Tree", HyPhy.THYPHY_TYPE_STRING, hyphyInstance).sData)
bl = retrieveValueByKey ("Branch lengths", HyPhy.THYPHY_TYPE_MATRIX, hyphyInstance)
print ("retrieved ", bl.mCols-1, "branch lengths")
for i in range(0,bl.mCols-1):
	print ("Branch ", i+1, " has length ", bl.MatrixCell(0,i))















#!/usr/bin/python

from distutils.core      import setup, Extension
from os                  import path
from glob                import glob
import sys


#incdir = get_python_inc(plat_specific=1)
#print incdir


#build the list of Source files

scriptPath = path.realpath(path.dirname(sys.argv[0]))
srcPath, libDir = path.split(scriptPath)
hyphyPath, srcDir = path.split(srcPath)
# with open('batchfiles.list') as fh:
#     resFiles = [(f, path.join(*(['..'] * 5 + f.split('/')))) for f in fh.read().split('\n') if f != '']

contribPath = path.join(hyphyPath, 'contrib')
sqlitePath = path.join(contribPath, 'SQLite-3.8.2')

linkPath = path.join(scriptPath, 'Link')
coreSrcPath = path.join(srcPath, 'core')
newSrcPath = path.join(srcPath, 'new')
guiSrcPath = path.join(srcPath, 'gui')
prefFile = [path.join(guiSrcPath, 'preferences.cpp')]

if sys.version_info >= (3,0,0):
    swigFile = [path.join(scriptPath, 'SWIGWrappers', 'THyPhy_py3.cpp')]
else:
    swigFile = [path.join(scriptPath, 'SWIGWrappers', 'THyPhy_python.cpp')]

coreSrcFiles = glob(path.join(coreSrcPath, '*.cpp'))
newSrcFiles = glob(path.join(newSrcPath, '*.cpp'))
sqliteFiles = glob(path.join(sqlitePath, '*.c'))
linkFiles = glob(path.join(linkPath, '*.cpp')) # + glob(path.join(linkPath, '*.cxx'))
utilFiles = glob(path.join(srcPath, 'utils', '*.cpp'))

sourceFiles = coreSrcFiles + newSrcFiles +  sqliteFiles + prefFile + linkFiles + swigFile + utilFiles

includePaths =  [path.join(p, 'include') for p in [coreSrcPath, newSrcPath, guiSrcPath]]
includePaths += [linkPath, contribPath, sqlitePath]

# check for 64bit and define as such
define_macros = [('__HYPHY_64__', None)] if '64' in architecture()[0] else []

# openmp on Mac OS X Lion is broken
#major, minor = mac_ver()[0].split('.')
#openmp = ['-fopenmp'] if int(major) < 10 or (int(major) == 10 and int(minor) < 7) else []

openmp = []

setup(
    name = 'HyPhy',
    version = '0.1.1',
    description = 'HyPhy package interface library',
    author = 'Sergei L Kosakovsky Pond',
    author_email = 'spond@temple.edu',
    url = 'http://www.hyphy.org/',
    packages = ['HyPhy'],
    package_dir = {'HyPhy': './'},
#    data_files = resFiles,
    # py_modules = ['HyPhy'],
    ext_modules = [Extension('_HyPhy',
            sourceFiles,
            include_dirs = includePaths,
            define_macros = [('SQLITE_PTR_SIZE','sizeof(long)'),
                             ('__UNIX__', None),
                             ('__MP__', None),
                             ('__MP2__', None),
                             ('_SLKP_LFENGINE_REWRITE_', None),
                             ('__AFYP_REWRITE_BGM__', None),
                             ('__HEADLESS__', None),
                             ('_HYPHY_LIBDIRECTORY_', '"/usr/local/lib/hyphy"')] + define_macros,
            libraries = ['pthread', 'ssl', 'crypto', 'curl'],
            extra_compile_args = [
                    '-Wno-int-to-pointer-cast',
                    # '-Wno-pointer-to-int-cast',
                    '-Wno-char-subscripts',
                    '-Wno-sign-compare',
                    '-Wno-parentheses',
                    '-Wno-uninitialized',
#                    '-Wno-conversion-null',
                    '-Wno-unused-variable',
#                    '-Wno-unused-but-set-variable',
                    '-Wno-shorten-64-to-32',
                    '-fsigned-char',
                    '-O3',
                    '-fpermissive',
                    '-fPIC',
            ] + openmp,
            extra_link_args = [
            ] + openmp
    )]
)
